from typing import Union
from pathlib import Path
import numpy

ndarray = numpy.ndarray

def get_human(image: ndarray, onnx_path: Union[str, Path] = './modnet_human_detect.onnx') -> ndarray: ...

def get_none_gray(image: ndarray, alpha: float = 1., s1: int = 9, s2: int = 5) -> ndarray: ...

def get_edge(image: ndarray, s: int = 9, score_thresh: float = 0.012) -> ndarray: ...

def get_sketch(image: ndarray, blur_size: int = 9, thresh_min: float = 0, thresh_max: float = 1.) -> ndarray: ...

def get_valid_area(image: ndarray, s: int = 3, std_thresh: float = 5., b: int = 13, k: int = 17) -> ndarray: ...
